//
//  ViewController.swift
//  busyness
//
//  Created by Micah Gomez on 8/21/20.
//

import UIKit

extension UserDefaults {
  func colorForKey(key: String) -> UIColor? {
	var colorReturnded: UIColor?
	if let colorData = data(forKey: key) {
	  do {
		if let color = try NSKeyedUnarchiver.unarchiveTopLevelObjectWithData(colorData) as? UIColor {
		  colorReturnded = color
		}
	  } catch {
		print("Error UserDefaults")
	  }
	}
	return colorReturnded
  }
  
  func setColor(color: UIColor?, forKey key: String) {
	var colorData: NSData?
	if let color = color {
	  do {
		let data = try NSKeyedArchiver.archivedData(withRootObject: color, requiringSecureCoding: false) as NSData?
		colorData = data
	  } catch {
		print("Error UserDefaults")
	  }
	}
	set(colorData, forKey: key)
  }
}

extension CIImage {
	/// Inverts the colors and creates a transparent image by converting the mask to alpha.
	/// Input image should be black and white.
	var transparent: CIImage? {
		return inverted?.blackTransparent
	}
	
	/// Inverts the colors.
	var inverted: CIImage? {
		guard let invertedColorFilter = CIFilter(name: "CIColorInvert") else { return nil }
		
		invertedColorFilter.setValue(self, forKey: "inputImage")
		return invertedColorFilter.outputImage
	}
	
	/// Converts all black to transparent.
	var blackTransparent: CIImage? {
		guard let blackTransparentFilter = CIFilter(name: "CIMaskToAlpha") else { return nil }
		blackTransparentFilter.setValue(self, forKey: "inputImage")
		return blackTransparentFilter.outputImage
	}
	
	/// Applies the given color as a tint color.
	func tinted(using color: UIColor) -> CIImage?
	{
		guard
			let transparentQRImage = transparent,
			let filter = CIFilter(name: "CIMultiplyCompositing"),
			let colorFilter = CIFilter(name: "CIConstantColorGenerator") else { return nil }
		
		let ciColor = CIColor(color: color)
		colorFilter.setValue(ciColor, forKey: kCIInputColorKey)
		let colorImage = colorFilter.outputImage
		
		filter.setValue(colorImage, forKey: kCIInputImageKey)
		filter.setValue(transparentQRImage, forKey: kCIInputBackgroundImageKey)
		
		return filter.outputImage!
	}
}

extension URL {
	
	/// Creates a QR code for the current URL in the given color.
	func qrImage(using color: UIColor) -> CIImage? {
		return qrImage?.tinted(using: color)
	}
	
	/// Returns a black and white QR code for this URL.
	var qrImage: CIImage? {
		guard let qrFilter = CIFilter(name: "CIQRCodeGenerator") else { return nil }
		let qrData = absoluteString.data(using: String.Encoding.ascii)
		qrFilter.setValue(qrData, forKey: "inputMessage")
		
		let qrTransform = CGAffineTransform(scaleX: 12, y: 12)
		return qrFilter.outputImage?.transformed(by: qrTransform)
	}
}

class ViewController: UIViewController, UIImagePickerControllerDelegate, UINavigationControllerDelegate, UIColorPickerViewControllerDelegate {
	
	//let prefs = UserDefaults(suiteName:"group.dev.mpg13.busyness")
	let defaults = UserDefaults(suiteName:"group.dev.mpg13.busyness")
	
	@IBOutlet var fullView: UIView!
	
	
	@IBOutlet var cardPhoto: UIButton!
	
	@IBOutlet var qrCodeImage: UIButton!
	
	@IBOutlet var nameTextField: UITextField!
	
	@IBOutlet var phoneTextField: UITextField!
	
	@IBOutlet var emailTextField: UITextField!
	
	let imagePicker = UIImagePickerController()
	let colorPicker = UIColorPickerViewController()
	
	override func viewDidLoad() {
		super.viewDidLoad()
		imagePicker.delegate = self
		colorPicker.delegate = self
		
		if(defaults?.colorForKey(key: "savedColor") != nil){
			fullView.backgroundColor = defaults?.colorForKey(key: "savedColor")
		}
		
		let qrURL = defaults?.string(forKey: "savedQRURL")
		if ((qrURL == nil) || (qrURL == "")) {
			let defaultQR = UIImage(systemName: "qrcode")
			qrCodeImage.setBackgroundImage(defaultQR, for: qrCodeImage.state)
		} else {
			let qrCode = URL(string: qrURL!)?.qrImage(using: UIColor.black)
			qrCodeImage.setBackgroundImage(UIImage(ciImage: qrCode!), for: qrCodeImage.state)
		}
		
		nameTextField.text = defaults?.string(forKey: "savedCardName")
		phoneTextField.text = defaults?.string(forKey: "savedCardPhone")
		emailTextField.text = defaults?.string(forKey: "savedCardEmail")
		if let cardImageData = defaults?.data(forKey: "savedCardPhoto"){
			cardPhoto.setBackgroundImage(UIImage(data: cardImageData), for: cardPhoto.state)}
		
		// Do any additional setup after loading the view.
	}
	
	@IBAction func nameTextFieldEntered(_ sender: Any) {
		defaults?.setValue(nameTextField.text, forKey: "savedCardName")
	}
	
	@IBAction func phoneTextFieldEntered(_ sender: Any) {
		defaults?.setValue(phoneTextField.text, forKey: "savedCardPhone")
	}
	
	@IBAction func emailTextFieldEntered(_ sender: Any) {
		defaults?.setValue(emailTextField.text, forKey: "savedCardEmail")
	}
	
	@IBAction func cardPhotoPressed(_ sender: Any) {
		imagePicker.allowsEditing = true
		imagePicker.sourceType = .photoLibrary
		
		present(imagePicker, animated: true, completion: nil)
	}
	
	@IBAction func qrCodeButtonPressed(_ sender: Any) {
		
		
		
		var alertController:UIAlertController?
		alertController = UIAlertController(title: "Enter a URL Below",
											message: "Text entered below will be turned into a QR code, which can be used as a quick link from your card.",
											preferredStyle: .alert)
		
		alertController!.addTextField(
			configurationHandler: {(textField: UITextField!) in
				textField.placeholder = "https://micahpgomez.dev"
			})
		
		let action = UIAlertAction(title: "Submit",
								   style: UIAlertAction.Style.default,
								   handler: {[weak self]
									(paramAction:UIAlertAction!) in
									if let textFields = alertController?.textFields{
										let theTextFields = textFields as [UITextField]
										let enteredText = theTextFields[0].text
										self?.defaults?.setValue(enteredText, forKey: "savedQRURL")
										
										if (enteredText == "") {
											let defaultQR = UIImage(systemName: "qrcode")
											self!.qrCodeImage.setBackgroundImage(defaultQR, for: self!.qrCodeImage.state)
										} else {
											let qrCode = URL(string: enteredText!)?.qrImage(using: UIColor.black)
											self!.qrCodeImage.setBackgroundImage(UIImage(ciImage: qrCode!), for: self!.qrCodeImage.state)
											self!.defaults?.setValue(UIImage(ciImage: qrCode!).pngData(), forKey: "savedQRPhoto")
										}
										
									}
								   })
		
		
		
		alertController?.addAction(UIAlertAction(title: "Cancel", style: .cancel, handler: nil))
		alertController?.addAction(action)
		self.present(alertController!,
					 animated: true,
					 completion: nil)
	}
	
	@IBAction func qrButtonLongPressed(_ sender: Any) {
		guard let url = URL(string: defaults?.string(forKey: "savedQRURL")! ?? "") else { return }
		UIApplication.shared.open(url)
	}
	
	@IBAction func colorPickerButton(_ sender: Any) {
		present(colorPicker, animated: true, completion: nil)
	}
	
	func imagePickerController(_ picker: UIImagePickerController, didFinishPickingMediaWithInfo info: [UIImagePickerController.InfoKey : Any]) {
		if let pickedImage = info[UIImagePickerController.InfoKey.editedImage] as? UIImage {
			cardPhoto.setBackgroundImage(pickedImage, for: cardPhoto.state)
			defaults?.setValue(pickedImage.pngData(), forKey: "savedCardPhoto")
		}
		
		dismiss(animated: true, completion: nil)
	}
	
	func imagePickerControllerDidCancel(_ picker: UIImagePickerController) {
		dismiss(animated: true, completion: nil)
	}
	
	func colorPickerViewControllerDidFinish(_ viewController: UIColorPickerViewController) {
		dismiss(animated: true, completion: nil)
	}
	
	func colorPickerViewControllerDidSelectColor(_ viewController: UIColorPickerViewController) {
		let color = viewController.selectedColor
		
		fullView.backgroundColor = color
		defaults?.setColor(color: color, forKey: "savedColor")
	}
	
}

